==> cds-integration-cloud-sdk-1.33.0-sources.jar: com/sap/cds/integration/cloudsdk/rest/client/JsonRestClient.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.integration.cloudsdk.rest.client;

import java.io.IOException;
import java.util.Map;

import org.apache.http.HttpRequest;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sap.cds.integration.cloudsdk.destination.DestinationResolver;
import com.sap.cds.services.environment.ServiceBinding;
import com.sap.cloud.sdk.cloudplatform.connectivity.DefaultHttpClientFactory;
import com.sap.cloud.sdk.cloudplatform.connectivity.HttpClientFactory;
import com.sap.cloud.sdk.cloudplatform.connectivity.HttpDestination;
import com.sap.cloud.sdk.cloudplatform.connectivity.OnBehalfOf;
import com.sap.cloud.sdk.cloudplatform.security.exception.TokenRequestDeniedException;
import com.sap.cloud.sdk.cloudplatform.security.exception.TokenRequestFailedException;

/**
 * Implementation of a simple JSON REST client.
 */
public class JsonRestClient {

	private static final Logger logger = LoggerFactory.getLogger(JsonRestClient.class);

	private static final String UTF_8			= "UTF-8";
	private static final String JSON_CONTENT	= "application/json";
	private static final String CONTENT_TYPE	= "content-type";

	protected final ObjectMapper mapper = new ObjectMapper();

	private final String apiUrl;
	private final CloseableHttpClient client;

	/**
	 * Creates the json rest client.
	 *
	 * @param apiUrl api url
	 * @param binding service binding
	 * @param onBehalfOf on behalf of {@link OnBehalfOf}
	 */
	public JsonRestClient(String apiUrl, ServiceBinding binding, OnBehalfOf onBehalfOf) {
		this(apiUrl, binding, 60000, 20, 2, onBehalfOf);
	}

	/**
	 * Creates the json rest client.
	 *
	 * @param apiUrl api url
	 * @param binding service binding
	 * @param timeoutMillis connection timeout
	 * @param maxConnections total max connections in pool
	 * @param maxConnectionPerRoute max connections per route
	 * @param onBehalfOf on behalf of {@link OnBehalfOf}
	 */
	public JsonRestClient(String apiUrl, ServiceBinding binding, long timeoutMillis, int maxConnections, int maxConnectionPerRoute, OnBehalfOf onBehalfOf) {
		this.apiUrl = apiUrl;

		DefaultHttpClientFactory.DefaultHttpClientFactoryBuilder builder = DefaultHttpClientFactory.builder();
		builder.timeoutMilliseconds((int) timeoutMillis);
		builder.maxConnectionsPerRoute(maxConnectionPerRoute);
		builder.maxConnectionsTotal(maxConnections);
		HttpClientFactory httpClientFactory = builder.build();
		HttpDestination dest = DestinationResolver.getDestinationForXsuaaBasedServiceBinding(apiUrl, binding, onBehalfOf);

		client = (CloseableHttpClient) httpClientFactory.createHttpClient(dest);
	}

	/**
	 * Performs a HTTP GET request and returns the JSON response.
	 *
	 * @param path destination URL path
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode getRequest(String path) throws IOException {
		HttpGet get = new HttpGet(getUrl(path));
		try {
			return handleJsonResponse(handleRequest(get));
		} finally {
			get.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP GET request and returns the response code only.
	 *
	 * @param path destination URL path
	 *
	 * @return response code
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 */
	public int getRequestWithOnlyResponseCode(String path) throws IOException {
		HttpGet get = new HttpGet(getUrl(path));
		try (CloseableHttpResponse response = handleRequest(get)) {
			return response.getStatusLine().getStatusCode();
		} finally {
			get.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP DELETE request and returns the JSON response.
	 *
	 * @param path destination URL path
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode deleteRequest(String path) throws IOException {
		HttpDelete del = new HttpDelete(getUrl(path));
		try {
			return handleJsonResponse(handleRequest(del));
		} finally {
			del.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP PUT request and returns the JSON response.
	 *
	 * @param path destination URL path
	 * @param data JSON request body
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode putRequest(String path, JsonNode data) throws IOException {
		HttpPut put = new HttpPut(getUrl(path));
		if (data != null) {
			put.setEntity(new StringEntity(mapper.writer().writeValueAsString(data), UTF_8));
		}
		try {
			return handleJsonResponse(handleRequest(put));
		} finally {
			put.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP POST request and returns the JSON response.
	 *
	 * @param path destination URL path
	 * @param data JSON request body
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode postRequest(String path, JsonNode data) throws IOException {
		String strData = mapper.writer().writeValueAsString(data);
		return postRequest(path, strData, null);
	}

	/**
	 * Performs a HTTP POST request and returns the JSON response.
	 *
	 * @param path destination URL path
	 * @param data plain string request body
	 * @param headers additional request headers
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode postRequest(String path, String data, Map<String, String> headers) throws IOException {
		HttpPost post = new HttpPost(getUrl(path));

		if (headers != null) {
			headers.forEach(post::setHeader);
		}

		if (data != null) {
			post.setEntity(new StringEntity(data, UTF_8));
		}
		try {
			return handleJsonResponse(handleRequest(post));
		} finally {
			post.releaseConnection();
		}
	}

	private CloseableHttpResponse handleRequest(HttpRequestBase request) throws IOException {
		setHeaders(request);
		try {
			return client.execute(request);
		} catch (TokenRequestDeniedException e) {
			throw new JsonRestClientResponseException(401, "Authentication invalid: " + e.getMessage());
		} catch (TokenRequestFailedException e) {
			throw new JsonRestClientResponseException(401, "Authentication failed: " + e.getMessage());
		}
	}


	/**
	 * Sets custom headers of the HTTP request.
	 *
	 * @param request HTTP request
	 */
	private void setHeaders(HttpRequest request) {

		logger.debug("Creating {} request to {}", request.getRequestLine().getMethod(), request.getRequestLine().getUri());

		request.removeHeaders(CONTENT_TYPE);
		request.addHeader(CONTENT_TYPE, JSON_CONTENT);
	}


	/**
	 * Handles HTTP response and parses the JSON content.
	 *
	 */
	private JsonNode handleJsonResponse(CloseableHttpResponse response) throws IOException {
		try (CloseableHttpResponse resp = response){
			int code = resp.getStatusLine().getStatusCode();

			logger.debug("Responded with status code '{}'", code);

			if (code >= 200 && code <= 207) {
				String contentType = JSON_CONTENT;
				if (resp.getEntity() != null) {
					if (resp.getEntity().getContentType() != null) {
						contentType = resp.getEntity().getContentType().getValue();
					}
					if (contentType.contains(JSON_CONTENT)) {
						String jsonData = EntityUtils.toString(resp.getEntity());
						return mapper.readValue(jsonData, JsonNode.class);
					} else {
						throw new IOException("Unexpected response format: Expected JSON but found '" + contentType + "'");
					}
				} else {
					return mapper.readValue("{}", JsonNode.class);
				}
			} else {
				String reason = resp.getStatusLine().getReasonPhrase();
				throw new JsonRestClientResponseException(code, "Unexpected request HTTP response (" + code + ") " + reason);
			}
		}
	}

	private String getUrl(String path) {
		return apiUrl + path;
	}
}

==> cds-integration-cloud-sdk-1.34.7-sources.jar: com/sap/cds/integration/cloudsdk/rest/client/JsonRestClient.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.integration.cloudsdk.rest.client;

import java.io.IOException;
import java.time.Duration;
import java.util.Map;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.apache.http.HttpRequest;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sap.cds.integration.cloudsdk.destination.DestinationResolver;
import com.sap.cds.integration.cloudsdk.destination.HttpClientProvider;
import com.sap.cds.services.environment.CdsProperties.ConnectionPool;
import com.sap.cds.services.environment.ServiceBinding;
import com.sap.cloud.sdk.cloudplatform.connectivity.OnBehalfOf;
import com.sap.cloud.sdk.cloudplatform.resilience.ResilienceRuntimeException;
import com.sap.cloud.sdk.cloudplatform.security.exception.TokenRequestDeniedException;
import com.sap.cloud.sdk.cloudplatform.security.exception.TokenRequestFailedException;

/**
 * Implementation of a simple JSON REST client.
 */
public class JsonRestClient {

	private static final Logger logger = LoggerFactory.getLogger(JsonRestClient.class);

	private static final String UTF_8			= "UTF-8";
	private static final String JSON_CONTENT	= "application/json";
	private static final String CONTENT_TYPE	= "content-type";

	protected final ObjectMapper mapper = new ObjectMapper();
	private final HttpClientProvider clientProvider;

	/**
	 * Creates the json rest client.
	 *
	 * @param apiUrl api url
	 * @param binding service binding
	 * @param onBehalfOf on behalf of {@link OnBehalfOf}
	 */
	public JsonRestClient(String apiUrl, ServiceBinding binding, OnBehalfOf onBehalfOf) {
		this(apiUrl, binding, new ConnectionPool(Duration.ofSeconds(60), 2, 20), onBehalfOf);
	}

	/**
	 * Creates the json rest client.
	 *
	 * @param apiUrl api url
	 * @param binding service binding
	 * @param connectionPool the {@link ConnectionPool} settings
	 * @param onBehalfOf on behalf of {@link OnBehalfOf}
	 */
	public JsonRestClient(String apiUrl, ServiceBinding binding, ConnectionPool connectionPool, OnBehalfOf onBehalfOf) {
		// passing runtime as null is okay, given the assumption that requests are always executed within an existing request context
		this.clientProvider = new HttpClientProvider(() -> {
			return DestinationResolver.getDestinationForXsuaaBasedServiceBinding(apiUrl, binding, onBehalfOf);
		}, connectionPool, null);
	}

	/**
	 * Performs a HTTP GET request and returns the JSON response.
	 *
	 * @param path destination URL path
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode getRequest(String path) throws IOException {
		HttpGet get = new HttpGet(path);
		try {
			return handleJsonResponse(handleRequest(get));
		} finally {
			get.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP GET request and returns the response code only.
	 *
	 * @param path destination URL path
	 *
	 * @return response code
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 */
	public int getRequestWithOnlyResponseCode(String path) throws IOException {
		HttpGet get = new HttpGet(path);
		try (CloseableHttpResponse response = handleRequest(get)) {
			return response.getStatusLine().getStatusCode();
		} finally {
			get.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP DELETE request and returns the JSON response.
	 *
	 * @param path destination URL path
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode deleteRequest(String path) throws IOException {
		HttpDelete del = new HttpDelete(path);
		try {
			return handleJsonResponse(handleRequest(del));
		} finally {
			del.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP PUT request and returns the JSON response.
	 *
	 * @param path destination URL path
	 * @param data JSON request body
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode putRequest(String path, JsonNode data) throws IOException {
		HttpPut put = new HttpPut(path);
		if (data != null) {
			put.setEntity(new StringEntity(mapper.writer().writeValueAsString(data), UTF_8));
		}
		try {
			return handleJsonResponse(handleRequest(put));
		} finally {
			put.releaseConnection();
		}
	}

	/**
	 * Performs a HTTP POST request and returns the JSON response.
	 *
	 * @param path destination URL path
	 * @param data JSON request body
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode postRequest(String path, JsonNode data) throws IOException {
		String strData = mapper.writer().writeValueAsString(data);
		return postRequest(path, strData, null);
	}

	/**
	 * Performs a HTTP POST request and returns the JSON response.
	 *
	 * @param path destination URL path
	 * @param data plain string request body
	 * @param headers additional request headers
	 *
	 * @return JSON response
	 *
	 * @throws IOException in case of a problem or the connection was aborted
	 * @throws JsonRestClientResponseException in case when the response HTTP is not 2xx
	 */
	public JsonNode postRequest(String path, String data, Map<String, String> headers) throws IOException {
		HttpPost post = new HttpPost(path);

		if (headers != null) {
			headers.forEach(post::setHeader);
		}

		if (data != null) {
			post.setEntity(new StringEntity(data, UTF_8));
		}
		try {
			return handleJsonResponse(handleRequest(post));
		} finally {
			post.releaseConnection();
		}
	}

	private CloseableHttpResponse handleRequest(HttpRequestBase request) throws IOException {
		setHeaders(request);
		try {
			return ((CloseableHttpClient) clientProvider.get()).execute(request);
		} catch (TokenRequestDeniedException | TokenRequestFailedException | ResilienceRuntimeException e) {
			if (ExceptionUtils.indexOfThrowable(e, TokenRequestDeniedException.class) >= 0) {
				throw new JsonRestClientResponseException(401, "Authentication invalid: " + e.getMessage());
			} else if (ExceptionUtils.indexOfThrowable(e, TokenRequestFailedException.class) >= 0) {
				throw new JsonRestClientResponseException(401, "Authentication failed: " + e.getMessage());
			}
			throw e;
		}
	}


	/**
	 * Sets custom headers of the HTTP request.
	 *
	 * @param request HTTP request
	 */
	private void setHeaders(HttpRequest request) {

		logger.debug("Creating {} request to {}", request.getRequestLine().getMethod(), request.getRequestLine().getUri());

		request.removeHeaders(CONTENT_TYPE);
		request.addHeader(CONTENT_TYPE, JSON_CONTENT);
	}


	/**
	 * Handles HTTP response and parses the JSON content.
	 *
	 */
	private JsonNode handleJsonResponse(CloseableHttpResponse response) throws IOException {
		try (CloseableHttpResponse resp = response){
			int code = resp.getStatusLine().getStatusCode();

			logger.debug("Responded with status code '{}'", code);

			if (code >= 200 && code <= 207) {
				String contentType = JSON_CONTENT;
				if (resp.getEntity() != null) {
					if (resp.getEntity().getContentType() != null) {
						contentType = resp.getEntity().getContentType().getValue();
					}
					if (contentType.contains(JSON_CONTENT)) {
						String jsonData = EntityUtils.toString(resp.getEntity());
						return mapper.readValue(jsonData, JsonNode.class);
					} else {
						throw new IOException("Unexpected response format: Expected JSON but found '" + contentType + "'");
					}
				} else {
					return mapper.readValue("{}", JsonNode.class);
				}
			} else {
				String reason = resp.getStatusLine().getReasonPhrase();
				throw new JsonRestClientResponseException(code, "Unexpected request HTTP response (" + code + ") " + reason);
			}
		}
	}
}
